@@ -298,6 +298,8 @@ def runlistpy(self, event):
     return dirs + pys
 
 
+greedy_cd_completer = False
+
 def cd_completer(self, event):
     relpath = event.symbol
     #print event # dbg
@@ -316,6 +318,9 @@ def cd_completer(self, event):
             return ents
         return []
 
+    if event.symbol.startswith('--'):
+        return ["--" + os.path.basename(d) for d in ip.user_ns['_dh']]
+    
     if relpath.startswith('~'):
         relpath = os.path.expanduser(relpath).replace('\\','/')
     found = []
@@ -344,7 +349,7 @@ def cd_completer(self, event):
             if d[-1] in ['/','\\']:
                 d = d[:-1]
 
-            subdirs = [p for p in os.listdir(d) if os.path.isdir( d + '/' + p)]
+            subdirs = [p for p in os.listdir(d) if os.path.isdir( d + '/' + p) and not p.startswith('.')]
             if subdirs:
                 matches = [ (d + '/' + p) for p in subdirs ]
                 return single_dir_expand(matches)
@@ -353,7 +358,10 @@ def cd_completer(self, event):
         else:
             return matches
 
-    return single_dir_expand(found)
+    if greedy_cd_completer:
+        return single_dir_expand(found)
+    else:
+        return found
 
 def apt_get_packages(prefix):
     out = os.popen('apt-cache pkgnames')
