@@ -24,16 +24,17 @@ __all__ = ['Inspector','InspectColors']
 
 # stdlib modules
 import __builtin__
+import StringIO
 import inspect
 import linecache
-import string
-import StringIO
-import types
 import os
+import string
 import sys
+import types
+
 # IPython's own
 from IPython import PyColorize
-from IPython.genutils import page,indent,Term,mkdict
+from IPython.genutils import page,indent,Term
 from IPython.Itpl import itpl
 from IPython.wildcard import list_namespace
 from IPython.ColorANSI import *
@@ -136,6 +137,7 @@ def getdoc(obj):
             ds = '%s\n%s' % (ds,ds2)
     return ds
 
+
 def getsource(obj,is_binary=False):
     """Wrapper around inspect.getsource.
 
@@ -155,7 +157,32 @@ def getsource(obj,is_binary=False):
     if is_binary:
         return None
     else:
-        return inspect.getsource(obj)
+        try:
+            src = inspect.getsource(obj)
+        except TypeError:
+            if hasattr(obj,'__class__'):
+                src = inspect.getsource(obj.__class__)
+        return src
+
+def getargspec(obj):
+    """Get the names and default values of a function's arguments.
+
+    A tuple of four things is returned: (args, varargs, varkw, defaults).
+    'args' is a list of the argument names (it may contain nested lists).
+    'varargs' and 'varkw' are the names of the * and ** arguments or None.
+    'defaults' is an n-tuple of the default values of the last n arguments.
+
+    Modified version of inspect.getargspec from the Python Standard
+    Library."""
+
+    if inspect.isfunction(obj):
+        func_obj = obj
+    elif inspect.ismethod(obj):
+        func_obj = obj.im_func
+    else:
+        raise TypeError, 'arg is not a Python function'
+    args, varargs, varkw = inspect.getargs(func_obj.func_code)
+    return args, varargs, varkw, func_obj.func_defaults
 
 #****************************************************************************
 # Class definitions
@@ -167,6 +194,7 @@ class myStringIO(StringIO.StringIO):
         self.write(*arg,**kw)
         self.write('\n')
 
+
 class Inspector:
     def __init__(self,color_table,code_color_table,scheme,
                  str_detail_level=0):
@@ -176,26 +204,6 @@ class Inspector:
         self.str_detail_level = str_detail_level
         self.set_active_scheme(scheme)
 
-    def __getargspec(self,obj):
-        """Get the names and default values of a function's arguments.
-
-        A tuple of four things is returned: (args, varargs, varkw, defaults).
-        'args' is a list of the argument names (it may contain nested lists).
-        'varargs' and 'varkw' are the names of the * and ** arguments or None.
-        'defaults' is an n-tuple of the default values of the last n arguments.
-
-        Modified version of inspect.getargspec from the Python Standard
-        Library."""
-
-        if inspect.isfunction(obj):
-            func_obj = obj
-        elif inspect.ismethod(obj):
-            func_obj = obj.im_func
-        else:
-            raise TypeError, 'arg is not a Python function'
-        args, varargs, varkw = inspect.getargs(func_obj.func_code)
-        return args, varargs, varkw, func_obj.func_defaults
-
     def __getdef(self,obj,oname=''):
         """Return the definition header for any callable object.
 
@@ -203,7 +211,7 @@ class Inspector:
         exception is suppressed."""
         
         try:
-            return oname + inspect.formatargspec(*self.__getargspec(obj))
+            return oname + inspect.formatargspec(*getargspec(obj))
         except:
             return None
  
@@ -278,7 +286,7 @@ class Inspector:
             self.noinfo('documentation',oname)
             return
         page(output)
-
+    
     def psource(self,obj,oname=''):
         """Print the source code for an object."""
 
@@ -293,23 +301,35 @@ class Inspector:
 
     def pfile(self,obj,oname=''):
         """Show the whole file where an object was defined."""
+
         try:
-            sourcelines,lineno = inspect.getsourcelines(obj)
+            try:
+                lineno = inspect.getsourcelines(obj)[1]
+            except TypeError:
+                # For instances, try the class object like getsource() does
+                if hasattr(obj,'__class__'):
+                    lineno = inspect.getsourcelines(obj.__class__)[1]
+                    # Adjust the inspected object so getabsfile() below works
+                    obj = obj.__class__
         except:
             self.noinfo('file',oname)
+            return
+
+        # We only reach this point if object was successfully queried
+        
+        # run contents of file through pager starting at line
+        # where the object is defined
+        ofile = inspect.getabsfile(obj)
+
+        if (ofile.endswith('.so') or ofile.endswith('.dll')):
+            print 'File %r is binary, not printing.' % ofile
+        elif not os.path.isfile(ofile):
+            print 'File %r does not exist, not printing.' % ofile
         else:
-            # run contents of file through pager starting at line
-            # where the object is defined
-            ofile = inspect.getabsfile(obj)
-            
-            if (ofile.endswith('.so') or ofile.endswith('.dll')):
-                print 'File %r is binary, not printing.' % ofile
-            elif not os.path.isfile(ofile):
-                print 'File %r does not exist, not printing.' % ofile
-            else:
-                # Print only text files, not extension binaries.
-                page(self.format(open(ofile).read()),lineno)
-            #page(self.format(open(inspect.getabsfile(obj)).read()),lineno)
+            # Print only text files, not extension binaries.  Note that
+            # getsourcelines returns lineno with 1-offset and page() uses
+            # 0-offset, so we must adjust.
+            page(self.format(open(ofile).read()),lineno-1)
 
     def pinfo(self,obj,oname='',formatter=None,info=None,detail_level=0):
         """Show detailed information about an object.
