@@ -40,18 +40,12 @@ except ImportError:
 # IPython imports
 import IPython
 from IPython import ultraTB, ipapi
+from IPython.Magic import Magic
 from IPython.genutils import Term,warn,error,flag_calls, ask_yes_no
 from IPython.iplib import InteractiveShell
 from IPython.ipmaker import make_IPython
-from IPython.Magic import Magic
 from IPython.ipstruct import Struct
-
-try: # Python 2.3 compatibility
-    set
-except NameError:
-    import sets
-    set = sets.Set
-
+from IPython.testing import decorators as testdec
 
 # Globals
 # global flag to pass around information about Ctrl-C without exceptions
@@ -66,6 +60,9 @@ MAIN_THREAD_ID = thread.get_ident()
 # Tag when runcode() is active, for exception handling
 CODE_RUN = None
 
+# Default timeout for waiting for multithreaded shells (in seconds)
+GUI_TIMEOUT = 10
+
 #-----------------------------------------------------------------------------
 # This class is trivial now, but I want to have it in to publish a clean
 # interface. Later when the internals are reorganized, code that uses this
@@ -359,12 +356,15 @@ class MTInteractiveShell(InteractiveShell):
     isthreaded = True
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
-                 user_ns=None,user_global_ns=None,banner2='',**kw):
+                 user_ns=None,user_global_ns=None,banner2='',
+                 gui_timeout=GUI_TIMEOUT,**kw):
         """Similar to the normal InteractiveShell, but with threading control"""
         
         InteractiveShell.__init__(self,name,usage,rc,user_ns,
                                   user_global_ns,banner2)
 
+        # Timeout we wait for GUI thread
+        self.gui_timeout = gui_timeout
 
         # A queue to hold the code to be executed. 
         self.code_queue = Queue.Queue()
@@ -408,11 +408,12 @@ class MTInteractiveShell(InteractiveShell):
             # Case 2
             return True
 
-        # shortcut - if we are in worker thread, or the worker thread is not running, 
-        # execute directly (to allow recursion and prevent deadlock if code is run early 
-        # in IPython construction)
+        # shortcut - if we are in worker thread, or the worker thread is not
+        # running, execute directly (to allow recursion and prevent deadlock if
+        # code is run early in IPython construction)
         
-        if (self.worker_ident is None or self.worker_ident == thread.get_ident()):
+        if (self.worker_ident is None
+            or self.worker_ident == thread.get_ident() ):
             InteractiveShell.runcode(self,code)
             return
 
@@ -423,7 +424,7 @@ class MTInteractiveShell(InteractiveShell):
         
         self.code_queue.put((code,completed_ev, received_ev))
         # first make sure the message was received, with timeout
-        received_ev.wait(5)
+        received_ev.wait(self.gui_timeout)
         if not received_ev.isSet():
             # the mainloop is dead, start executing code directly
             print "Warning: Timeout for mainloop thread exceeded"
@@ -510,7 +511,7 @@ class MatplotlibShellBase:
     Given Python's MRO, this should be used as the FIRST class in the
     inheritance hierarchy, so that it overrides the relevant methods."""
     
-    def _matplotlib_config(self,name,user_ns):
+    def _matplotlib_config(self,name,user_ns,user_global_ns=None):
         """Return items needed to setup the user's shell with matplotlib"""
 
         # Initialize matplotlib to interactive mode always
@@ -564,17 +565,28 @@ class MatplotlibShellBase:
         self.pylab.draw_if_interactive = flag_calls(self.pylab.draw_if_interactive)
 
         # Build a user namespace initialized with matplotlib/matlab features.
-        user_ns = IPython.ipapi.make_user_ns(user_ns)
+        user_ns, user_global_ns = IPython.ipapi.make_user_namespaces(user_ns,
+            user_global_ns)
 
-        exec ("import matplotlib\n"
-              "import matplotlib.pylab as pylab\n") in user_ns
+        # Import numpy as np/pyplot as plt are conventions we're trying to
+        # somewhat standardize on.  Making them available to users by default
+        # will greatly help this. 
+        exec ("import numpy\n"
+              "import numpy as np\n"
+              "import matplotlib\n"
+              "import matplotlib.pylab as pylab\n"
+              "try:\n"
+              "    import matplotlib.pyplot as plt\n"
+              "except ImportError:\n"
+              "    pass\n"
+              ) in user_ns
         
         # Build matplotlib info banner
         b="""
   Welcome to pylab, a matplotlib-based Python environment.
   For more information, type 'help(pylab)'.
 """
-        return user_ns,b
+        return user_ns,user_global_ns,b
 
     def mplot_exec(self,fname,*where,**kw):
         """Execute a matplotlib script.
@@ -596,7 +608,8 @@ class MatplotlibShellBase:
         # if a backend switch was performed, reverse it now
         if self.mpl_use._called:
             self.matplotlib.rcParams['backend'] = self.mpl_backend
-        
+
+    @testdec.skip_doctest
     def magic_run(self,parameter_s=''):
         Magic.magic_run(self,parameter_s,runner=self.mplot_exec)
 
@@ -614,7 +627,7 @@ class MatplotlibShell(MatplotlibShellBase,InteractiveShell):
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
                  user_ns=None,user_global_ns=None,**kw):
-        user_ns,b2 = self._matplotlib_config(name,user_ns)
+        user_ns,user_global_ns,b2 = self._matplotlib_config(name,user_ns,user_global_ns)
         InteractiveShell.__init__(self,name,usage,rc,user_ns,user_global_ns,
                                   banner2=b2,**kw)
 
@@ -623,7 +636,7 @@ class MatplotlibMTShell(MatplotlibShellBase,MTInteractiveShell):
 
     def __init__(self,name,usage=None,rc=Struct(opts=None,args=None),
                  user_ns=None,user_global_ns=None, **kw):
-        user_ns,b2 = self._matplotlib_config(name,user_ns)
+        user_ns,user_global_ns,b2 = self._matplotlib_config(name,user_ns,user_global_ns)
         MTInteractiveShell.__init__(self,name,usage,rc,user_ns,user_global_ns,
                                     banner2=b2,**kw)
 
